;==================================================
;   S M B 3     G O A L     C A R D
;   by Mathos
;
;   A sprite that's intented to be a replica of the goal card in SMB3. Highly customizable.
;   As with most level ending sprites, setting the extra bit in LM activates the secret exit.
;
;==================================================
!ExtraBits = $7FAB10 ; Don't touch this.
!ItemSRAM = $1F2B  ; - 3 consecutive bytes of RAM used to store the item data
                   ;   By default, it uses the 3 empty adresses saved to SRAM at $1F2B so you can use it without any patch
				   ;   But if you already use these, you'll have to use 3 other consecutives bytes
                   ;   and save them with the SRAM+ patch (http://www.smwcentral.net/?p=section&a=details&id=12940)
!GFXBeg = $04    ; \
!GFXEnd = $05    ; | Scratch RAM used temporarly to draw the sprite's graphics. Most "empty" adresses are sufficient.
!GFXNum = $06    ; / You'd better leave it as it is, unless you already use them for some reason.
!ScratchMusic = $00   ; - Scratch RAM used to temporraly store which music to play
!Timer = #$3F      ; - How many frames before changing powerups, minus 1. Any (2^n)-1 value supported.
!PowerNum = #$07   ; - How many frames in the following table, minus 1. Any (2^n)-1 value supported.
PowerTable: ; $00 = Mushroom
            ; $01 = Flower
            ; $02 = Feather
            ; $03 = Star
db $00,$01,$02,$03,$00,$01,$02,$03
!DisappearFX = #$03 ; Which effect to use when sprite is collected:
                    ; #$00 => None
					; #$01 => Spinjump effect (little stars)
					; #$02 => Puff of smoke
					; #$03 => Both of the above
!DisappearSound = #$05    ; \ SFX to play when item is touched and disappears, with SFX bank
!DisappearSBank = $1DF9  ; /
!MusicNormal = #$0C ; - Music to play when you finished getting a 3-item set
!MusicGotIt = #$0B  ; - Music to play if the items match and you win one of them
!LeastPrize = #$01  ; #$00 => Mario doesn't get anything if the items don't match
                    ; #$01 => He gets at least a regular 1UP
!SendToInventory = #$00	; If this is set to any non-zero value,
!RoutineROM = $ABCDEF   ;  the item will be sent to MarioE's OW inventory.
                        ;  http://www.smwcentral.net/?p=section&a=details&id=10629
                        ;  In this case, you need to put the output ROM address for the routine that add items.
                        ; Otherwise, it is placed in the item box.

						
						

print "INIT ",pc
STZ $1570,x
STZ $1594,x
RTL

print "MAIN ",pc
	PHB				;\
	PHK				; | Change the data bank to the one our code is running from.
	PLB				; | This is a good practice.
	JSR SpriteCode	; | Jump to the sprite's function.
	PLB				;/ Restore old data bank.
RTL		; And RETURN.

;===================================
;Sprite Function
;===================================

Return:
	RTS
	
SpriteCode:
	LDA $14C8,x			;\
	CMP #$08			; | If sprite dead,
	BNE Return			;/ RETURN.
	LDA $9D				;\
	BNE Return			;/ If locked, RETURN.
	
	JSR SubOffScreen_X3	; handle offscreen
	
	JSR Graphics
	
	LDA $1570,x		; Check if sprite has already been touched
	CMP #$01
	BEQ GoalClear
	JSL $01A7DC		; Check if contact
	BCC NoContact
Contact:
	LDA #$01
	STA $1570,x
	BRA Return
GoalClear:
	JSR GoalClear_1
	BRA Return
NoContact:
	JSR NoContact_1
	BRA Return
	
	;==O==O==O==

NoContact_1:
	LDA $14
	AND !Timer
	BNE NoContact_5
	LDA $1594,x
	INC
	AND !PowerNum
	STA $1594,x
NoContact_5:	
	RTS
	
	;==O==O==O==
	
GoalClear_1:
	JSL $00FA8F
	LDY $010A            ; what item needs to be changed
	LDA !ItemSRAM,y	
	AND #$C0
	BEQ ChangeItem1
	CMP #$40
	BEQ ChangeItem2
	CMP #$80
	BEQ ChangeItem3
ChangeItem1:            ; change first item
	LDA #$40
	STA $00
		PHX
		LDA $1594,x
		TAX
		LDA PowerTable,x
		PLX
	ORA $00
	STA !ItemSRAM,y
	LDA !MusicNormal
	STA !ScratchMusic
	JMP Disappear
ChangeItem2:            ; change second item
	LDA !ItemSRAM,y
	AND #$03
	STA $02
	LDA #$80
	STA $00
		PHX
		LDA $1594,x
		TAX
		LDA PowerTable,x
		PLX
	CLC : ROL : ROL
	ORA $00
	ORA $02
	STA !ItemSRAM,y
	LDA !MusicNormal
	STA !ScratchMusic
	JMP Disappear
ChangeItem3:            ; change third item
	LDA !ItemSRAM,y
	AND #$0F
	STA $02
	LDA #$C0
	STA $00
		PHX
		LDA $1594,x
		TAX
		LDA PowerTable,x
		PLX
	CLC : ROL : ROL : ROL : ROL
	ORA $00
	ORA $02
	STA !ItemSRAM,y
TestBrelan:             ; are all items identical ?
	PHX
	LDA !ItemSRAM,y
	AND #$3F
	BEQ GotMushroom
	CMP #$15
	BEQ GotFlower
	CMP #$2A
	BEQ GotFeather
	CMP #$3F
	BEQ GotStar
	BRA Just1UP
GotMushroom:
	LDX #$01
	BRA Finalize
GotFlower:
	LDX #$02
	BRA Finalize
GotFeather:
	LDX #$04
	BRA Finalize
GotStar:
	LDX #$03
	BRA Finalize
Just1UP:
	LDA !MusicNormal
	STA !ScratchMusic
	PLX
	LDA #$00
	STA !ItemSRAM,y
	LDA !LeastPrize
	BEQ Disappear
	LDA #$01
	STA $18E4
	BRA Disappear
Finalize:                   ;after getting powerup number, actually collect it
	LDA !MusicGotIt
	STA !ScratchMusic
	LDA !SendToInventory
	BNE SendIt
	TXA
	STA $0DC2
	PLX
	BRA Disappear
	SendIt:
	TXA
	JSL !RoutineROM
	LDA #$00
	STA !ItemSRAM,y
	PLX
Disappear:             ; sprite "dies"
	LDA !DisappearFX
	BEQ FXNone
	AND #$01
	BEQ NoSpinFX
	JSL $07FC3B
	NoSpinFX:
	LDA !DisappearFX
	AND #$02
	BEQ FXNone
	JSR FXSmoke_0
FXNone:
	LDA !DisappearSound
	STA !DisappearSBank
	STZ $14C8,x
	LDA #$FF                ; \ set normal exit
    STA $1493               ; /
    STA $0DDA               ; set music
	LDA !ExtraBits,x
	AND #$04
	BEQ NormalExit
	LDA #$01
	STA $141C
	NormalExit:
	LDA !ScratchMusic
    STA $1DFB
	RTS

FXSmoke_0:			; subroutine that draws smoke (thanks to whoever made it!)
LDY #$03
FXSmoke_1:
	LDA $17C0,y
	BEQ FXSmoke_2
	DEY
	BPL FXSmoke_1
	RTS
FXSmoke_2:
	LDA #$01
	STA $17C0,y
	LDA #$1C     
	STA $17CC,y
	LDA $D8,x
	STA $17C4,y
	LDA $E4,x
	STA $17C8,y
	RTS
	
	
	
;===================================
;Graphics Code (CUSTOM, NOT OPTIMIZED)
;===================================

; =FRAME TABLE=

PROPS:	db $38,$3A,$34,$34
TILES:	db $24,$26,$0E,$48
XDISP:	db $00,$00,$00,$00
YDISP:	db $00,$00,$00,$00

Graphics:
	JSR GET_DRAW_INFO

	PHX		; Let's push X in order to not destroy everything.

	;===Actual frame drawing decision here===
		LDA $1594,x
		TAX
		LDA PowerTable,x
		STA !GFXBeg
		STA !GFXEnd
	; ===End of decision with an interval===

	LDA !GFXEnd
	SEC
	SBC !GFXBeg
	STA !GFXNum
	INC !GFXEnd
	LDX !GFXBeg
LOOP:
	LDA $00			; Application of X displacement
	CLC
	ADC XDISP,x
	STA $0300,y

	LDA $01			; Application of Y displacement
	CLC
	ADC YDISP,x
	STA $0301,y

	LDA TILES,x		; Set tile to use
	STA $0302,y

	LDA PROPS,x		; Set PROPERTIES.
	STA $0303,y

	INY			;\
	INY			; | The OAM is 8x8, but our sprite is 16x16 ..
	INY			; | So increment it 4 times.
	INY			;/
	
	INX
	CPX !GFXEnd
	BCC LOOP
	
	PLX			; Pull back the sprite index! We pushed it at the beginning of the routine.

	LDY #$02			; Y ends with the tile size .. 02 means it's 16x16
	LDA !GFXNum

	JSL $01B7B3		; Call the routine that draws the sprite.
	RTS				; Never forget this!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;       Y = index to sprite OAM ($300)
;       $00 = sprite x position relative to screen boarder
;       $01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

SPR_T1:              db $0C,$1C
SPR_T2:              db $01,$02

GET_DRAW_INFO:
	STZ $186C,x             ; reset sprite offscreen flag, vertical
	STZ $15A0,x             ; reset sprite offscreen flag, horizontal
	LDA $E4,x               ; \
	CMP $1A                 ;  | set horizontal offscreen if necessary
	LDA $14E0,x             ;  |
	SBC $1B                 ;  |
	BEQ .ON_SCREEN_X        ;  |
	INC $15A0,x             ; /

.ON_SCREEN_X
	LDA $14E0,x             ; \
	XBA                     ;  |
	LDA $E4,x               ;  |
	REP #$20                ;  |
	SEC                     ;  |
	SBC $1A                 ;  | mark sprite invalid if far enough off screen
	CLC                     ;  |
	ADC.w #$0040            ;  |
	CMP.w #$0180            ;  |
	SEP #$20                ;  |
	ROL A                   ;  |
	AND #$01                ;  |
	STA $15C4,x             ; / 
	BNE .INVALID           ; 
	
	LDY #$00                ; \ set up loop:
	LDA $1662,x             ;  | 
	AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
	BEQ .ON_SCREEN_LOOP     ;  | else, go through loop once
	INY                     ; / 
.ON_SCREEN_LOOP
	LDA $D8,x               ; \ 
	CLC                     ;  | set vertical offscreen if necessary
	ADC SPR_T1,y            ;  |
	PHP                     ;  |
	CMP $1C                 ;  | (vert screen boundry)
	ROL $00                 ;  |
	PLP                     ;  |
	LDA $14D4,x             ;  | 
	ADC #$00                ;  |
	LSR $00                 ;  |
	SBC $1D                 ;  |
	BEQ .ON_SCREEN_Y        ;  |
	LDA $186C,x             ;  | (vert offscreen)
	ORA SPR_T2,y            ;  |
	STA $186C,x             ;  |
.ON_SCREEN_Y
	DEY                     ;  |
	BPL .ON_SCREEN_LOOP     ; /

	LDY $15EA,x             ; get offset to sprite OAM
	LDA $E4,x               ; \ 
	SEC                     ;  | 
	SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
	STA $00                 ; / 
	LDA $D8,x               ; \ 
	SEC                     ;  | 
	SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
	STA $01                 ; / 
	RTS	 ; return

.INVALID
	PLA                     ; \ return from *main gfx routine* subroutine...
	PLA                     ;  |    ...(not just this subroutine)
	RTS                     ; /
	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; SUB_OFF_SCREEN
; This subroutine deals with sprites that have moved off screen
; It is adapted from the subroutine at $01AC0D
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                   
SPR_T12:            db $40,$C0
SPR_T13:            db $01,$FE
SPR_T14:            db $30,$C0,$A0,$C0,$A0,$F0,$60,$90                    ;bank 1 sizes
                                db $30,$C0,$A0,$80,$A0,$40,$60,$B0                    ;bank 3 sizes
SPR_T15:            db $01,$FF,$01,$FF,$01,$FF,$01,$FF                    ;bank 1 sizes
                                                db $01,$FF,$01,$FF,$01,$00,$01,$FF                    ;bank 3 sizes
 
SubOffScreen_X1:  LDA #$02                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | (table entry to use on horizontal levels)
SubOffScreen_X2:  LDA #$04                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X3:  LDA #$06                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X4:  LDA #$08                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X5:  LDA #$0A                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X6:  LDA #$0C                ;  |
                    BRA STORE_03            ;  |
SubOffScreen_X7:  LDA #$0E                ;  |
STORE_03:                    STA $03                                            ;  |            
                                                BRA START_SUB                           ;  |
SubOffScreen_X0:  STZ $03                                               ; /
 
START_SUB:          JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_35           ; /
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ; |
                    BNE VERTICAL_LEVEL      ; /    
                    LDA $D8,x               ; \
                    CLC                     ; |
                    ADC #$50                ; | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ; | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ; |
                    CMP #$02                ; |
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0756 VC:176 00 FL:205
                    AND #$01                ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0780 VC:176 00 FL:205
                    ORA $03                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0796 VC:176 00 FL:205
                    STA $01                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0820 VC:176 00 FL:205
                    TAY                     ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0844 VC:176 00 FL:205
                    LDA $1A                 ;A:8A01 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizcHC:0858 VC:176 00 FL:205
                    CLC                     ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0882 VC:176 00 FL:205
                    ADC SPR_T14,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZcHC:0896 VC:176 00 FL:205
                    ROL $00                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizcHC:0928 VC:176 00 FL:205
                    CMP $E4,x               ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:0966 VC:176 00 FL:205
                    PHP                     ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:0996 VC:176 00 FL:205
                    LDA $1B                 ;A:8AC0 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizCHC:1018 VC:176 00 FL:205
                    LSR $00                 ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdiZCHC:1042 VC:176 00 FL:205
                    ADC SPR_T15,y           ;A:8A00 X:0009 Y:0001 D:0000 DB:01 S:01F0 P:envMXdizcHC:1080 VC:176 00 FL:205
                    PLP                     ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F0 P:eNvMXdizcHC:1112 VC:176 00 FL:205
                    SBC $14E0,x             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1140 VC:176 00 FL:205
                    STA $00                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1172 VC:176 00 FL:205
                    LSR $01                 ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:eNvMXdizCHC:1196 VC:176 00 FL:205
                    BCC SPR_L31             ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1234 VC:176 00 FL:205
                    EOR #$80                ;A:8AFF X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdiZCHC:1250 VC:176 00 FL:205
                    STA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1266 VC:176 00 FL:205
SPR_L31:            LDA $00                 ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1290 VC:176 00 FL:205
                    BPL RETURN_35           ;A:8A7F X:0009 Y:0001 D:0000 DB:01 S:01F1 P:envMXdizCHC:1314 VC:176 00 FL:205
ERASE_SPRITE:       LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ; |
                    BCC KILL_SPRITE         ; /  
                    LDY $161A,x             ;A:FF08 X:0007 Y:0001 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1108 VC:059 00 FL:2878
                    CPY #$FF                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZCHC:1140 VC:059 00 FL:2878
                    BEQ KILL_SPRITE         ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1156 VC:059 00 FL:2878
                    LDA #$00                ;A:FF08 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdizcHC:1172 VC:059 00 FL:2878
                    STA $1938,y             ;A:FF00 X:0007 Y:0000 D:0000 DB:01 S:01F3 P:envMXdiZcHC:1188 VC:059 00 FL:2878
KILL_SPRITE:        STZ $14C8,x             ; erase sprite
RETURN_35:          RTS                     ; return
 
VERTICAL_LEVEL:     LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ; |
                    BNE RETURN_35           ; /
                    LDA $13                 ; \
                    LSR A                   ; |
                    LSR A
                    BCS RETURN_35           ; /
 
                    LDA $E4,x               ; \
                    CMP #$00                ;  | if the sprite has gone off the side of the level...
                    LDA $14E0,x             ;  |
                    SBC #$00                ;  |
                    CMP #$02                ;  |
                    BCS ERASE_SPRITE        ; /  ...erase the sprite
                    LDA $13                 ;A:0000 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:1218 VC:250 00 FL:5379
                    LSR A                   ;A:0016 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1242 VC:250 00 FL:5379
                    AND #$01                ;A:000B X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1256 VC:250 00 FL:5379
                    STA $01                 ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1272 VC:250 00 FL:5379
                    TAY                     ;A:0001 X:0009 Y:00E4 D:0000 DB:01 S:01F3 P:envMXdizcHC:1296 VC:250 00 FL:5379
                    LDA $1C                 ;A:001A X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0052 VC:251 00 FL:5379
                    CLC                     ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0076 VC:251 00 FL:5379
                    ADC SPR_T12,y           ;A:00BD X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0090 VC:251 00 FL:5379
                    ROL $00                 ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:enVMXdizCHC:0122 VC:251 00 FL:5379
                    CMP $D8,x               ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0160 VC:251 00 FL:5379
                    PHP                     ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0190 VC:251 00 FL:5379
                    LDA.w $001D             ;A:006D X:0009 Y:0001 D:0000 DB:01 S:01F2 P:eNVMXdizcHC:0212 VC:251 00 FL:5379
                    LSR $00                 ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdiZcHC:0244 VC:251 00 FL:5379
                    ADC SPR_T13,y           ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:enVMXdizCHC:0282 VC:251 00 FL:5379
                    PLP                     ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F2 P:envMXdiZCHC:0314 VC:251 00 FL:5379
                    SBC $14D4,x             ;A:0000 X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNVMXdizcHC:0342 VC:251 00 FL:5379
                    STA $00                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0374 VC:251 00 FL:5379
                    LDY $01                 ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0398 VC:251 00 FL:5379
                    BEQ SPR_L38             ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0422 VC:251 00 FL:5379
                    EOR #$80                ;A:00FF X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0438 VC:251 00 FL:5379
                    STA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0454 VC:251 00 FL:5379
SPR_L38:            LDA $00                 ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0478 VC:251 00 FL:5379
                    BPL RETURN_35           ;A:007F X:0009 Y:0001 D:0000 DB:01 S:01F3 P:envMXdizcHC:0502 VC:251 00 FL:5379
                    BMI ERASE_SPRITE        ;A:8AFF X:0002 Y:0000 D:0000 DB:01 S:01F3 P:eNvMXdizcHC:0704 VC:184 00 FL:5490
 
SUB_IS_OFF_SCREEN:
                    LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0
                    ORA $186C,x             ; |  
                    RTS                     ; / return